/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef BRUSHEDITOR_H
#define BRUSHEDITOR_H

#include "Group.h"
#include "Scrollbar.h"
#include "Texture.h"
#include "Toggle.h"
#include <list>

namespace SharpConstruct
{
	namespace Interface
	{
		const unsigned THMB_SZ = 32;

		class TextureSwatch : public Toggle
		{
		public:
			TextureSwatch( std::string path );
			
			virtual ~TextureSwatch()
			{}

			Texture& Txtr();

			sigc::signal< void, TextureSwatch* >& ClickSignal();
		private:
			void _click();
			
			Texture _txtr;
			sigc::signal< void, TextureSwatch* > _click_signal;
		};

		class BrushEditor : public HorizontalGroup
		{
		public:
			BrushEditor();
			virtual ~BrushEditor();

			bool AddAlpha( std::string path, bool show_msgs = true );

			Texture& CurrentAlpha();

			sigc::signal< void >& ShowOpenDialogSignal();
			sigc::signal< void, Texture& >& AlphaChangedSignal();
		protected:
			void _draw();
		private:
			const unsigned _GRD_SZ;

			void _load_default_brushset();

			void _swatch_click_handler( TextureSwatch* t );
			void _scroll_handler();
			void _remove_handler();

			VerticalGroup _controls;
			Button _add;
			Button _remove;
			HorizontalGroup _all;
			std::vector< VerticalGroup > _cols;
			VerticalScrollbar _scroll;

			TextureSwatch* _cur;

			std::list< TextureSwatch* > _alphas;

			sigc::signal< void > _show_open_dialog_signal;
			sigc::signal< void, Texture& > _alpha_changed_signal;
		};
	}
}
#endif // BRUSHEDITOR_H
