/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef DEFORMDIALOG_H
#define DEFORMDIALOG_H

#include "Button.h"
#include "Dialog.h"
#include "Mesh.h"
#include "Slider.h"
#include "Toggle.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class MainWindow;
		
		class DeformDialog : public Dialog
		{
		public:
			DeformDialog( MainWindow& mw );
				
			virtual ~DeformDialog()
			{}
		private:
			void _make_copy();
			void _revert_to_copy( float );
			void _transfer( float );
			void _prepare_change( float );
		
			void _preview_handler();
			void _cb_move( float v );
			void _cb_scale( float v );
			void _cb_deflate( float v );
			void _cb_smooth( float v );
			void _cb_flood( float v );
			void _cb_subdivide();
			void _cb_optimize();
			void _cb_mirror();
			void _cb_flip_normals();
			void _cb_unify();
			void _cb_to_triangles();
		
			MainWindow& _parent;
			Mesh _mesh_copy;
		
			HorizontalGroup _axes;
			Toggle _xaxis;
			Toggle _yaxis;
			Toggle _zaxis;
			VerticalGroup _slide_defs;
			Toggle _preview;
			Slider _move;
			Slider _scale;
			Slider _deflate;
			Slider _smooth;
			Slider _flood;
			HorizontalGroup _sub_group;
			Button _subdivide;
			Toggle _sub_smooth;
			Button _optimize;
			Button _mirror;
			Button _flip_normals;
			Button _unify;
			Button _to_triangles;

			Button _pt;
			void _cb_pt();
		};
	}
}
#endif // DEFORMDIALOG_H
