/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Repeater.h"
#include "WidgetList.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

Repeater::Repeater()
: _original( 0 )
{
	MouseDownSignal.connect( sigc::mem_fun( *this, &Repeater::_mouse_down ) );
	MouseMoveSignal.connect( sigc::mem_fun( *this, &Repeater::_mouse_move ) );
	MouseUpSignal.connect( sigc::mem_fun( *this, &Repeater::_mouse_up ) );
}

void Repeater::MouseDown()
{
	if( _original == 0 )
		return;
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Repeater::MouseUp()
{
	if( _original == 0 )
		return;	
	if( Active() && Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Repeater::MouseMove()
{
	if( _original == 0 )
		return;
	if( !Contains( Event::Mouse.Location ) )
	{
		Hide();
		_original = 0;
		MakeUnfocused();
	}
	else if( Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}

void Repeater::SetOriginal( Widget& w )
{
	_original = &w;
	SetW( w.W() );
	SetH( w.H() );
	SetX( Event::Mouse.Location.X() - W() / 2 );
	SetY( Event::Mouse.Location.Y() - H() / 2 );
	SetCursor( w.Cursor() );
}

void Repeater::_draw()
{
	if( _original == 0 )
		return;

	const Math::Point2D< int > o( X() - _original->X(),
				      Y() - _original->Y() );

	glTranslatef( o.X(), o.Y(), 0 );
	_fake_data();
	_original->Draw();
	_unfake_data();
	glTranslatef( -o.X(), -o.Y(), 0 );
}

void Repeater::_fake_data()
{
	_old_mdownloc = Event::Mouse.DownLocation;
	_old_mloc = Event::Mouse.Location;
	Event::Mouse.Location.SetX( Event::Mouse.Location.X() - X() + _original->X() );
	Event::Mouse.Location.SetY( Event::Mouse.Location.Y() - Y() + _original->Y() );
	Event::Mouse.DownLocation.SetX( Event::Mouse.DownLocation.X() - X() + _original->X() );
	Event::Mouse.DownLocation.SetY( Event::Mouse.DownLocation.Y() - Y() + _original->Y() );
	//std::cout << _original->X() << " " << Event::Mouse.Location.X() << std::endl;
}
void Repeater::_unfake_data()
{
	Event::Mouse.Location = _old_mloc;
	Event::Mouse.DownLocation = _old_mdownloc;
}

void Repeater::_mouse_down()
{
	_fake_data();
	_original->MouseDownSignal();
	_unfake_data();
}
void Repeater::_mouse_move()
{
	_fake_data();
	_original->MouseMoveSignal();
	_unfake_data();
}
void Repeater::_mouse_up()
{
	_fake_data();
	_original->MouseUpSignal();
	_unfake_data();
}
