/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "TextureDisplay.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

TextureDisplay::TextureDisplay( const Math::Point2D< int >& link, Math::Size2D< int > s )
	: _loc_link( &link ), _size( s ), _visible( true ), _txtr( 0 )
{}

void TextureDisplay::SetOffset( Math::Point2D< int > o )
{
	_offset = o;
}

bool TextureDisplay::Visible() const
{
	return _visible;
}
void TextureDisplay::SetVisible( bool v )
{
	_visible = v;
}

void TextureDisplay::SetTexture( Texture& t )
{
	_txtr = &t;
}

void TextureDisplay::Draw()
{
	if( Visible() && _txtr )
	{
		glEnable( GL_TEXTURE_2D );

		_txtr->Bind();
		
		const int x = _loc_link->X() + _offset.X();
		const int y = _loc_link->Y() + _offset.Y();
		glBegin( GL_QUADS );
		{
			glTexCoord2f( 0, 0 );
			glVertex2f( x, y );

			glTexCoord2f( 1, 0 );
			glVertex2f( x + _size.W(), y );

			glTexCoord2f( 1, 1 );
			glVertex2f( x + _size.W(), y + _size.H() );

			glTexCoord2f( 0, 1 );
			glVertex2f( x, y + _size.H() );
		} glEnd();

		glDisable( GL_TEXTURE_2D );
	}
}
