/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Interface.h"
#include "Text.h"
#include "Tooltip.h"
#include "TopWindow.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

Tooltip::Tooltip( const Math::Point2D< int >& l )
: _loc_link( &l )
{}

std::string Tooltip::Message() const
{
	std::string t = _message;
	if( Event::Keyboard.ControlDown && _extended.length() > 0 )
		t += " NEWLINE NEWLINE " + _extended;
	return t;
}
void Tooltip::SetMessage( std::string m, std::string e )
{
	_message = m;
	_extended = e;
}

void Tooltip::Draw()
{
	unsigned max_sz = 500; // In pixels
	if( max_sz > TopWindow::Display().W() )
		max_sz = TopWindow::Display().W();

	Math::Rect2D< int > bounds( Event::Mouse.Location.X() + 10,
				    Event::Mouse.Location.Y() + 10, 0, 0 );

	std::vector< std::string > words;
	std::vector< std::string > lines( 1 );
	std::stringstream ss;
	std::string tmp;
	unsigned ln = 0;
	ss.str( Message() );
	while( ss >> tmp )
		words.push_back( tmp );
	for( unsigned i = 0; i < words.size(); ++i )
	{
		if( words[ i ] == "NEWLINE" )
		{
			lines.push_back( "" );
			++ln;
		}
		else
		{
			if( ( lines[ ln ].length() + words[ i ].length() ) *
			    GLOut::glout.CurrentFont().W() > max_sz - 4 )
			{
				lines.push_back( "" );
				++ln;
			}
			
			lines[ ln ] += words[ i ] + " ";

			if( bounds.W() < lines[ ln ].length() * GLOut::glout.CurrentFont().W() )
				bounds.SetW( lines[ ln ].length() * GLOut::glout.CurrentFont().W() );
		}
	}

	// Insure the tooltip doesn't move off the screen in the lower-right corner
	if( bounds.Right() > TopWindow::Display().W() )
		bounds.SetX( TopWindow::Display().W() - bounds.W() );
	if( bounds.Bottom() > TopWindow::Display().H() )
		bounds.SetY( TopWindow::Display().H() - bounds.H() );
	// Insure the tooltip doesn't move off the screen in the upper-left corner
	if( bounds.X() < 0 )
		bounds.SetX( 0 );
	if( bounds.Y() < 0 )
		bounds.SetY( 0 );
	bounds.SetH( ( GLOut::glout.CurrentFont().H() + 4 ) * lines.size() + 2 );

	glColor4f( 0.3, 0.3, 0.3, 0.3 );
	glRecti( bounds.X(), bounds.Y(),
		 bounds.Right() + 2, bounds.Bottom() + 2 );
	glColor4f( 1.0, 0.8, 0.3, 0.8 );
	glRecti( bounds.X() + 2, bounds.Y() + 2,
		 bounds.Right(), bounds.Bottom() );
	glColor3f( 0, 0, 0 );

	unsigned y = bounds.Y() + GLOut::glout.CurrentFont().H() / 2 + 2;
	for( unsigned i = 0; i < lines.size(); i++ )
	{
		GLOut::glout << GLFormat::Align::Left()
			     << GLFormat::Location( bounds.X() + 4, y )
			     << lines[ i ];
		y += GLOut::glout.CurrentFont().H() + 4;
	}
}
